package com.gci.boot.modules.dataview.service;

import com.gci.boot.modules.dataview.vo.VehicleCenterVo;

import java.text.ParseException;
import java.util.List;
import java.util.Map;

/**
 * @author lyz
 * @date 2023/9/27 11:47
 */
public interface VehiclesCenterService {

    /**
     * 违章统计
     * @param startTime  开始日期
     * @param endTime  结束日期
     * @param groupType 分组类型 1-项目 2-车型
     * @return
     */
    Map violationStatistics(String startTime, String endTime, Integer groupType) throws ParseException;

    /**
     * 保险统计
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param groupType 分组类型 1-项目 2-车型
     * @return
     * @throws ParseException
     */
    Map insuranceStatistics(String startTime, String endTime, Integer groupType) throws ParseException;

    /**
     * 事故统计
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param groupType 分组类型 1-项目 2-车型
     * @return
     * @throws ParseException
     */
    Map accidentStatistics(String startTime, String endTime, Integer groupType) throws ParseException;



    /**
     * 年检统计
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param groupType 分组类型 1-项目 2-车型
     * @return
     * @throws ParseException
     */
    Map inspectionStatistics(String startTime, String endTime, Integer groupType) throws ParseException;








    /**
     * 维修统计
     * @param startTimeStr  开始日期
     * @param endTimeStr  结束日期
     * @return
     */
    Map repairStatistics(String startTimeStr, String endTimeStr) throws ParseException;

    /**
     * 保养统计
     * @param startTimeStr  开始日期
     * @param endTimeStr  结束日期
     * @return
     */
    Map upkeepStatistics(String startTimeStr, String endTimeStr) throws ParseException;
}
